# SPDX-License-Identifier: GPL-2.0+
# This file is auto generated do not edit by hand

import os, sys
import ctypes

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

import pytest

SO = f"./tests/t_params_modules.{gf.lib_ext()}"
MOD = "./tests/params_modules.mod"

x = gf.fFort(SO, MOD)


class Test_params_modules:
    def test_check_ints(self):
        assert x.int_i1_0 == -1

        with pytest.raises(AttributeError) as cm:
            x.int_i1_0 = -99

        assert x.int_i1_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.int_i1_1 = -99

        assert x.int_i1_2 == 1

        with pytest.raises(AttributeError) as cm:
            x.int_i1_2 = -99

        assert x.int_i2_0 == -1

        with pytest.raises(AttributeError) as cm:
            x.int_i2_0 = -99

        assert x.int_i2_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.int_i2_1 = -99

        assert x.int_i2_2 == 1

        with pytest.raises(AttributeError) as cm:
            x.int_i2_2 = -99

        assert x.int_i4_0 == -1

        with pytest.raises(AttributeError) as cm:
            x.int_i4_0 = -99

        assert x.int_i4_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.int_i4_1 = -99

        assert x.int_i4_2 == 1

        with pytest.raises(AttributeError) as cm:
            x.int_i4_2 = -99

        assert x.int_i8_0 == -1

        with pytest.raises(AttributeError) as cm:
            x.int_i8_0 = -99

        assert x.int_i8_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.int_i8_1 = -99

        assert x.int_i8_2 == 1

        with pytest.raises(AttributeError) as cm:
            x.int_i8_2 = -99

    def test_check_reals(self):
        assert x.real_r4_0 == -3.140000104904175

        with pytest.raises(AttributeError) as cm:
            x.real_r4_0 = -99.9

        assert x.real_r4_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.real_r4_1 = -99.9

        assert x.real_r4_2 == 3.140000104904175

        with pytest.raises(AttributeError) as cm:
            x.real_r4_2 = -99.9

        assert x.real_r8_0 == -3.140000104904175

        with pytest.raises(AttributeError) as cm:
            x.real_r8_0 = -99.9

        assert x.real_r8_1 == 0

        with pytest.raises(AttributeError) as cm:
            x.real_r8_1 = -99.9

        assert x.real_r8_2 == 3.140000104904175

        with pytest.raises(AttributeError) as cm:
            x.real_r8_2 = -99.9

    def test_check_logicals(self):
        assert not x.logicals_0

        with pytest.raises(AttributeError) as cm:
            x.logicals_0 = True

        assert x.logicals_1

        with pytest.raises(AttributeError) as cm:
            x.logicals_1 = False

    def test_check_ints_1d(self):
        assert np.allclose(x.int_i1_1d, np.array([-10, -1, 0, 1, 10]))

        with pytest.raises(AttributeError) as cm:
            x.int_i1_1d = np.array([1, 2, 3])

        assert np.allclose(x.int_i2_1d, np.array([-10, -1, 0, 1, 10]))

        with pytest.raises(AttributeError) as cm:
            x.int_i2_1d = np.array([1, 2, 3])

        assert np.allclose(x.int_i4_1d, np.array([-10, -1, 0, 1, 10]))

        with pytest.raises(AttributeError) as cm:
            x.int_i4_1d = np.array([1, 2, 3])

        assert np.allclose(x.int_i8_1d, np.array([-10, -1, 0, 1, 10]))

        with pytest.raises(AttributeError) as cm:
            x.int_i8_1d = np.array([1, 2, 3])

    def test_check_reals_1d(self):
        assert np.allclose(
            x.real_r4_1d, np.array([-3.140000104904175, 0.0, 3.140000104904175])
        )

        with pytest.raises(AttributeError) as cm:
            x.real_r4_1d = np.array([1, 2, 3])

        assert np.allclose(
            x.real_r8_1d, np.array([-3.140000104904175, 0.0, 3.140000104904175])
        )

        with pytest.raises(AttributeError) as cm:
            x.real_r8_1d = np.array([1, 2, 3])

    def test_check_logicals_1d(self):
        assert np.allclose(x.logicals_0_1d, np.array([True, False, True, False]))

    def test_check_ints_2d(self):
        assert np.allclose(
            x.int_i1_2d, np.array([-10, -1, 0, 1, 10, 50]).reshape(2, 3, order="F")
        )

        with pytest.raises(AttributeError) as cm:
            x.int_i1_2d = np.array([1, 2, 3])

        assert np.allclose(
            x.int_i2_2d, np.array([-10, -1, 0, 1, 10, 50]).reshape(2, 3, order="F")
        )

        with pytest.raises(AttributeError) as cm:
            x.int_i2_2d = np.array([1, 2, 3])

        assert np.allclose(
            x.int_i4_2d, np.array([-10, -1, 0, 1, 10, 50]).reshape(2, 3, order="F")
        )

        with pytest.raises(AttributeError) as cm:
            x.int_i4_2d = np.array([1, 2, 3])

        assert np.allclose(
            x.int_i8_2d, np.array([-10, -1, 0, 1, 10, 50]).reshape(2, 3, order="F")
        )

        with pytest.raises(AttributeError) as cm:
            x.int_i8_2d = np.array([1, 2, 3])

    def test_check_reals_2d(self):
        assert np.allclose(
            x.real_r4_2d,
            np.array(
                [
                    -6.28000020980835,
                    -3.140000104904175,
                    0.0,
                    1.1111,
                    3.140000104904175,
                    6.28000020980835,
                ]
            ).reshape(2, 3, order="F"),
        )

        with pytest.raises(AttributeError) as cm:
            x.real_r4_2d = np.array([1, 2, 3])

        assert np.allclose(
            x.real_r8_2d,
            np.array(
                [
                    -6.28000020980835,
                    -3.140000104904175,
                    0.0,
                    1.1111,
                    3.140000104904175,
                    6.28000020980835,
                ]
            ).reshape(2, 3, order="F"),
        )

        with pytest.raises(AttributeError) as cm:
            x.real_r8_2d = np.array([1, 2, 3])

    def test_check_logicals_2d(self):
        assert np.allclose(
            x.logicals_0_2d,
            np.array([True, False, True, False, True, False]).reshape(2, 3, order="F"),
        )

    def test_check_cmplx(self):
        assert x.complex_r4_0 == complex(-3.140000104904175, -3.140000104904175)

        with pytest.raises(AttributeError) as cm:
            x.complex_r4_0 = -99.9

        assert x.complex_r4_1 == complex(0.0, 0.0)

        with pytest.raises(AttributeError) as cm:
            x.complex_r4_1 = -99.9

        assert x.complex_r4_2 == complex(3.140000104904175, 3.140000104904175)

        with pytest.raises(AttributeError) as cm:
            x.complex_r4_2 = -99.9

        assert x.complex_r8_0 == complex(-3.140000104904175, -3.140000104904175)

        with pytest.raises(AttributeError) as cm:
            x.complex_r8_0 = -99.9

        assert x.complex_r8_1 == complex(0.0, 0.0)

        with pytest.raises(AttributeError) as cm:
            x.complex_r8_1 = -99.9

        assert x.complex_r8_2 == complex(3.140000104904175, 3.140000104904175)

        with pytest.raises(AttributeError) as cm:
            x.complex_r8_2 = -99.9

    def test_check_cmplx_1d(self):
        assert np.allclose(
            x.complex_r4_1d,
            np.array(
                [
                    complex(-3.140000104904175, -3.140000104904175),
                    complex(0.0, 0.0),
                    complex(3.140000104904175, 3.140000104904175),
                ]
            ),
        )

        with pytest.raises(AttributeError) as cm:
            x.complex_r4_1d = np.array([1, 2, 3])

        assert np.allclose(
            x.complex_r8_1d,
            np.array(
                [
                    complex(-3.140000104904175, -3.140000104904175),
                    complex(0.0, 0.0),
                    complex(3.140000104904175, 3.140000104904175),
                ]
            ),
        )

        with pytest.raises(AttributeError) as cm:
            x.complex_r8_1d = np.array([1, 2, 3])

    def test_check_cmplx_2d(self):
        assert np.allclose(
            x.complex_r4_2d,
            np.array(
                [
                    complex(-6.28000020980835, -6.28000020980835),
                    complex(-3.140000104904175, -3.140000104904175),
                    complex(0.0, 0.0),
                    complex(0.0, -1.0),
                    complex(3.140000104904175, 3.140000104904175),
                    complex(-6.28000020980835, -6.28000020980835),
                ]
            ).reshape(2, 3, order="F"),
        )

        with pytest.raises(AttributeError) as cm:
            x.complex_r4_2d = np.array([1, 2, 3])

        assert np.allclose(
            x.complex_r8_2d,
            np.array(
                [
                    complex(-6.28000020980835, -6.28000020980835),
                    complex(-3.140000104904175, -3.140000104904175),
                    complex(0.0, 0.0),
                    complex(0.0, -1.0),
                    complex(3.140000104904175, 3.140000104904175),
                    complex(-6.28000020980835, -6.28000020980835),
                ]
            ).reshape(2, 3, order="F"),
        )

        with pytest.raises(AttributeError) as cm:
            x.complex_r8_2d = np.array([1, 2, 3])



# SPDX-License-Identifier: GPL-2.0+

import os, sys
import ctypes
from pprint import pprint

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

import pytest

SO = f"./tests/oo.{gf.lib_ext()}"
MOD = "./tests/oo.mod"

x = gf.fFort(SO, MOD)


@pytest.mark.skip
class TestOOMethods:
    def test_p_proc_call(self):
        x.p_proc.proc_no_pass = x.func_dt_no_pass
        y = x.p_proc.proc_no_pass(1)
        y2 = x.func_dt_no_pass(1)

        assert y.result == y2.result



from utils import *


modname = "params_modules"
libname = "t_params_modules"
filename_f90 = "../t_params_modules.f90"
filename_py = "../t_params_modules_test.py"


def create_ints():
    fort_strs = []
    py_strs = []
    values = [-1, 0, 1]

    for n, k in fort_int_kinds.items():
        for indv, v in enumerate(values):
            name = f"int_{n}_{indv}"

            fort_strs.append(
                fort_param.substitute(
                    type="integer", kind=n, var=name, value=f"{v}_{n}"
                )
            )

            py_strs.append(py_value_comp.substitute(name=name, value=v))
            py_strs.append(py_fail.substitute(var=name, bad=-99))

    return fort_strs, py_strs


def create_ints_array_1d():
    fort_strs = []
    py_strs = []
    values = np.array([-10, -1, 0, 1, 10])
    v = f"{make_array_values(values)}"
    vstr = f"np.array([{make_array_values(values)}])"
    for n, k in fort_int_kinds.items():
        name = f"int_{n}_1d"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="integer", kind=n, var=name, shape=len(values), value=v
            )
        )

        py_strs.append(py_array_check.substitute(name=name, value=vstr))
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def create_ints_array_2d():
    fort_strs = []
    py_strs = []
    values = np.array([-10, -1, 0, 1, 10, 50]).reshape(2, 3)
    v = f"{make_array_values(values)}"
    vstr = f'np.array([{make_array_values(values)}]).reshape(2,3, order="F")'
    for n, k in fort_int_kinds.items():
        name = f"int_{n}_2d"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="integer", kind=n, var=name, shape="2,3", value=v
            )
        )

        py_strs.append(py_array_check.substitute(name=name, value=vstr))
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def create_reals():
    fort_strs = []
    py_strs = []
    values = [-3.140000104904175, 0, 3.140000104904175]

    for n, k in fort_real_kinds.items():
        for indv, v in enumerate(values):
            name = f"real_{n}_{indv}"

            fort_strs.append(
                fort_param.substitute(type="real", kind=n, var=name, value=f"{v}_{n}")
            )

            py_strs.append(py_value_comp.substitute(name=name, value=v))
            py_strs.append(py_fail.substitute(var=name, bad=-99.9))

    return fort_strs, py_strs


def create_reals_array_1d():
    fort_strs = []
    py_strs = []
    values = np.array([-3.140000104904175, 0, 3.140000104904175])
    v = f"{make_array_values(values)}"
    vstr = f"np.array([{make_array_values(values)}])"

    for n, k in fort_real_kinds.items():
        name = f"real_{n}_1d"
        v = f"{make_array_values(values,n)}"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="real", kind=n, var=name, shape=len(values), value=v
            )
        )

        py_strs.append(py_array_check.substitute(name=name, value=vstr))
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def create_reals_array_2d():
    fort_strs = []
    py_strs = []
    values = np.array(
        [
            -2 * 3.140000104904175,
            -3.140000104904175,
            0.0,
            1.1111,
            3.140000104904175,
            2 * 3.140000104904175,
        ]
    ).reshape(2, 3)
    vstr = f'np.array([{make_array_values(values)}]).reshape(2,3, order="F")'
    for n, k in fort_real_kinds.items():
        name = f"real_{n}_2d"
        v = f"{make_array_values(values,n)}"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="real", kind=n, var=name, shape="2,3", value=v
            )
        )

        py_strs.append(py_array_check.substitute(name=name, value=vstr))
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def create_logicals():
    fort_strs = []
    py_strs = []
    fvalues = [".false.", ".true."]
    pyvalues = [False, True]

    n = "logical"
    for indv, v in enumerate(fvalues):
        name = f"logicals_{indv}"

        fort_strs.append(fort_bool_param.substitute(var=name, value=f"{v}"))

        if v == ".true.":
            py_strs.append(py_bool_true.substitute(name=name))
            py_strs.append(py_fail.substitute(var=name, bad=False))
        else:
            py_strs.append(py_bool_false.substitute(name=name))
            py_strs.append(py_fail.substitute(var=name, bad=True))

    return fort_strs, py_strs


def create_logicals_1d():
    fort_strs = []
    py_strs = []
    values = np.array([True, False, True, False])
    v = f".true., .false., .true., .false."
    vstr = f"np.array([{make_array_values(values)}])"

    n = "logical"
    name = f"logicals_0_1d"

    fort_strs.append(
        fort_bool_param_array_multi.substitute(var=name, shape="4", value=v)
    )

    py_strs.append(py_array_check.substitute(name=name, value=vstr))

    return fort_strs, py_strs


def create_logicals_2d():
    fort_strs = []
    py_strs = []
    values = np.array([True, False, True, False, True, False]).reshape(2, 3)
    v = f".true., .false., .true., .false., .true., .false."
    vstr = f'np.array([{make_array_values(values)}]).reshape(2,3, order="F")'

    n = "logical"
    name = f"logicals_0_2d"

    fort_strs.append(
        fort_bool_param_array_multi.substitute(var=name, shape="2,3", value=v)
    )

    py_strs.append(py_array_check.substitute(name=name, value=vstr))

    return fort_strs, py_strs


def create_cmplx():
    fort_strs = []
    py_strs = []

    for n, k in fort_real_kinds.items():
        values = [
            f"(-3.140000104904175_{n},-3.140000104904175_{n})",
            f"(0_{n},0_{n})",
            f"(3.140000104904175_{n},3.140000104904175_{n})",
        ]
        pyvalues = [
            "complex(-3.140000104904175,-3.140000104904175)",
            "complex(0.0,0.0)",
            "complex(3.140000104904175,3.140000104904175)",
        ]
        for indv, v in enumerate(values):
            name = f"complex_{n}_{indv}"

            fort_strs.append(
                fort_param.substitute(type="complex", kind=n, var=name, value=v)
            )

            py_strs.append(py_value_comp.substitute(name=name, value=pyvalues[indv]))
            py_strs.append(py_fail.substitute(var=name, bad=-99.9))

    return fort_strs, py_strs


def create_cmplx_array_1d():
    fort_strs = []
    py_strs = []

    for n, k in fort_real_kinds.items():
        name = f"complex_{n}_1d"
        # Kind of a component is different to the kind of the overall complex variable so skip setting the kind to make things easier
        v = f"(-3.140000104904175,-3.140000104904175), (0,0), (3.140000104904175,3.140000104904175)"
        pyvalues = "complex(-3.140000104904175,-3.140000104904175), complex(0.0,0.0), complex(3.140000104904175,3.140000104904175)"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="complex", kind=n, var=name, shape=3, value=v
            )
        )

        py_strs.append(
            py_array_check.substitute(name=name, value=f"np.array([{pyvalues}])")
        )
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def create_cmplx_array_2d():
    fort_strs = []
    py_strs = []

    for n, k in fort_real_kinds.items():
        name = f"complex_{n}_2d"

        v = f"(-6.28000020980835,-6.28000020980835), (-3.140000104904175,-3.140000104904175), (0,0),(0,-1), (3.140000104904175,3.140000104904175),(-6.28000020980835,-6.28000020980835)"

        pyvalues = "complex(-6.28000020980835,-6.28000020980835), complex(-3.140000104904175,-3.140000104904175), complex(0.0,0.0),complex(0.0,-1.0), complex(3.140000104904175,3.140000104904175), complex(-6.28000020980835,-6.28000020980835)"

        fort_strs.append(
            fort_param_array_multi.substitute(
                type="complex", kind=n, var=name, shape="2,3", value=v
            )
        )

        py_strs.append(
            py_array_check.substitute(
                name=name, value=f'np.array([{pyvalues}]).reshape(2,3, order="F")'
            )
        )
        py_strs.append(py_fail.substitute(var=name, bad="np.array([1,2,3])"))

    return fort_strs, py_strs


def make_tests(file_f90, file_py, name, func):
    f, py = func()
    write_lines(file_f90, f, 1)
    write_line(file_py, py_proc.substitute(function=name), 1)
    write_lines(file_py, py, 2)


with open(filename_f90, "w") as file_f90, open(filename_py, "w") as file_py:
    write_line(file_f90, fort_module_start.substitute(name=modname), 0)
    write_line(file_py, py_module_start.substitute(modname=modname, libname=libname), 0)

    make_tests(file_f90, file_py, "check_ints", create_ints)
    make_tests(file_f90, file_py, "check_reals", create_reals)
    make_tests(file_f90, file_py, "check_logicals", create_logicals)

    make_tests(file_f90, file_py, "check_ints_1d", create_ints_array_1d)
    make_tests(file_f90, file_py, "check_reals_1d", create_reals_array_1d)
    make_tests(file_f90, file_py, "check_logicals_1d", create_logicals_1d)

    make_tests(file_f90, file_py, "check_ints_2d", create_ints_array_2d)
    make_tests(file_f90, file_py, "check_reals_2d", create_reals_array_2d)
    make_tests(file_f90, file_py, "check_logicals_2d", create_logicals_2d)

    make_tests(file_f90, file_py, "check_cmplx", create_cmplx)
    make_tests(file_f90, file_py, "check_cmplx_1d", create_cmplx_array_1d)
    make_tests(file_f90, file_py, "check_cmplx_2d", create_cmplx_array_2d)

    # print(fort_module_mid.substitute(),file=file_f90)
    write_line(file_f90, fort_module_end.substitute(name=modname), 0)



# SPDX-License-Identifier: GPL-2.0+

import os, sys
import ctypes
from pprint import pprint

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

import pytest

SO = f"./tests/common.{gf.lib_ext()}"
MOD = "./tests/com.mod"

x = gf.fFort(SO, MOD)


class TestCommonBlocks:
    def test_set_values(self):
        x.x_int = 1
        assert x.x_int == 1

    def test_get_comm(self):
        assert len(x._module.common)



import string
import textwrap
import random
import numpy as np

from dataclasses import dataclass

import builtins
import contextlib


INDENT = " " * 4

py_proc = string.Template(
    textwrap.dedent(
        """
            def test_${function}(self):
            """
    ).strip()
)

py_proc_call = string.Template(
    textwrap.dedent(
        """
            result = x.${proc}(${args})
            """
    ).strip()
)

py_proc_return = string.Template(
    textwrap.dedent(
        """
            assert result.result
            """
    ).strip()
)

py_proc_arg_check = string.Template(
    textwrap.dedent(
        """
            assert result.args['${name}'] == ${value}
            """
    ).strip()
)

py_proc_arg_array_check = string.Template(
    textwrap.dedent(
        """
            assert np.array_equal(result.args['${name}'], ${value})
            """
    ).strip()
)

py_proc_stdout = string.Template(
    textwrap.dedent(
        """
            def test_${function}(self,capfd):
            """
    ).strip()
)

py_capture_stdout = string.Template(
    textwrap.dedent(
        """
            out, err = capfd.readouterr()
            """
    ).strip()
)

py_comp_stdout = string.Template(
    textwrap.dedent(
        """
            assert out.replace('\n','') == ${value}
            """
    ).strip()
)

py_value_set = string.Template(
    textwrap.dedent(
        """
            x.${name} = ${value}
            """
    ).strip()
)

py_value_comp = string.Template(
    textwrap.dedent(
        """
            assert x.${name} == ${value}
            """
    ).strip()
)

py_fail = string.Template(
    textwrap.dedent(
        """with pytest.raises(AttributeError) as cm:
                x.${var} = ${bad}
            """
    )
)

py_array_check = string.Template(
    textwrap.dedent(
        """
            assert np.allclose(x.${name}, ${value})
            """
    ).strip()
)

py_bool_true = string.Template(
    textwrap.dedent(
        """
            assert x.${name}
            """
    ).strip()
)

py_bool_false = string.Template(
    textwrap.dedent(
        """
            assert not x.${name}
            """
    ).strip()
)

fort_param = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),parameter :: ${var} = ${value}
"""
    ).strip()
)

fort_bool_param = string.Template(
    textwrap.dedent(
        """
    logical,parameter :: ${var} = ${value}
"""
    ).strip()
)

fort_bool_var = string.Template(
    textwrap.dedent(
        """
    logical :: ${var}
"""
    ).strip()
)

fort_param_array_sing = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),parameter,dimension(${shape}) :: ${var} = ${value}
"""
    ).strip()
)

fort_param_array_multi = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),parameter,dimension(${shape}) :: ${var} = reshape( (/ ${value}/), shape(${var}))
"""
    ).strip()
)

fort_bool_param_array_multi = string.Template(
    textwrap.dedent(
        """
    logical,parameter,dimension(${shape}) :: ${var} = reshape( (/ ${value}/), shape(${var}))
"""
    ).strip()
)


fort_set_var = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}) :: ${var} = ${value}
"""
    ).strip()
)

fort_set_array_sing = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}) :: ${var} = ${value}
"""
    ).strip()
)

fort_set_array_multi = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}) :: ${var} = reshape( (/ ${value}/), shape(${var}))
"""
    ).strip()
)


fort_var = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}) :: ${var}
"""
    ).strip()
)

fort_array = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}) :: ${var} 
"""
    ).strip()
)


fort_var_opt = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),optional :: ${var}
"""
    ).strip()
)

fort_array_opt = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}),optional :: ${var} 
"""
    ).strip()
)


fort_var_value = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),value :: ${var}
"""
    ).strip()
)

fort_array_value = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}),value :: ${var} 
"""
    ).strip()
)


fort_array_allocatable = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}),allocatable :: ${var} 
"""
    ).strip()
)

fort_array_pointer = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}),pointer :: ${var} 
"""
    ).strip()
)

fort_array_target = string.Template(
    textwrap.dedent(
        """
    ${type}(${kind}),dimension(${shape}),target :: ${var} 
"""
    ).strip()
)


fort_func_start = string.Template(
    textwrap.dedent(
        """
    function ${name}(${args}) result(${result}) 
        implicit none
"""
    ).strip()
)

fort_func_end = string.Template(
    textwrap.dedent(
        """
    end function ${name}
"""
    ).strip()
)


fort_sub_start = string.Template(
    textwrap.dedent(
        """
    subroutine ${name}(${args}) 
        implicit none
"""
    ).strip()
)

fort_sub_end = string.Template(
    textwrap.dedent(
        """
    end subroutine ${name}
"""
    ).strip()
)


fort_var_set = string.Template(
    textwrap.dedent(
        """
    ${name} = ${value}
"""
    ).strip()
)

fort_var_comp = string.Template(
    textwrap.dedent(
        """
    if(${name}/=${value}) return
"""
    ).strip()
)


fort_var_write = string.Template(
    textwrap.dedent(
        """
   write(*,*) ${name}
"""
    ).strip()
)

fort_module_start = string.Template(
    textwrap.dedent(
        """
! SPDX-License-Identifier: GPL-2.0+
! This file is auto generated do not edit by hand
module ${name}
    implicit none

    integer, parameter :: r4 = selected_real_kind(4)
    integer, parameter :: r8 = selected_real_kind(8)
    integer, parameter :: r16 = selected_real_kind(32)

    integer, parameter :: i1 = selected_int_kind(1)
    integer, parameter :: i2 = selected_int_kind(2)
    integer, parameter :: i4 = selected_int_kind(4)
    integer, parameter :: i8 = selected_int_kind(8)
                                            
    integer, parameter :: c1 = selected_char_kind ("ascii")
    integer, parameter :: c4  = selected_char_kind ('ISO_10646')
"""
    )
)


fort_module_mid = string.Template(
    textwrap.dedent(
        """

contains

"""
    )
)


fort_module_end = string.Template(
    textwrap.dedent(
        """
end module ${name}
"""
    )
)


py_module_start = string.Template(
    textwrap.dedent(
        """
# SPDX-License-Identifier: GPL-2.0+
# This file is auto generated do not edit by hand

import os, sys
import ctypes

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

import pytest

SO = f"./tests/${libname}.{gf.lib_ext()}"
MOD = "./tests/${modname}.mod"

x = gf.fFort(SO, MOD)

class Test_${modname}:
"""
    )
)


fort_int_kinds = {
    "i1": 1,
    "i2": 2,
    "i4": 4,
    "i8": 8,
}

fort_real_kinds = {
    "r4": 4,
    "r8": 8,
    #    'r16' :32,
}

fort_char_kinds = {
    "c1": 1,
    "c4": 4,
}


def make_array_values(array, kind=None):
    if kind is not None:
        sep = f"_{kind}"
    else:
        sep = ""

    return ", ".join([f"{i}{sep}" for i in np.asfortranarray(array).flatten()])


def write_lines(file, lines, indent_level):
    for line in lines:
        write_line(file, line, indent_level)


def write_line(file, line, indent_level):
    try:
        if line.startswith("\n"):
            line = line[1:]
    except AttributeError:
        pass
    print(f"{indent_level*INDENT}{line} \n", file=file)



# SPDX-License-Identifier: GPL-2.0+
# This file is auto generated do not edit by hand

import os, sys
import ctypes

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

import pytest

SO = f"./tests/t_vars_modules.{gf.lib_ext()}"
MOD = "./tests/vars_modules.mod"

x = gf.fFort(SO, MOD)


class Test_vars_modules:
    def test_check_ints(self):
        assert x.int_i1_0 == -1

        x.int_i1_0 = -2

        assert x.int_i1_0 == -2

        result = x.check_int_i1_0()

        assert result.result

        assert x.int_i1_1 == 0

        x.int_i1_1 = 0

        assert x.int_i1_1 == 0

        result = x.check_int_i1_1()

        assert result.result

        assert x.int_i1_2 == 1

        x.int_i1_2 = 2

        assert x.int_i1_2 == 2

        result = x.check_int_i1_2()

        assert result.result

        assert x.int_i2_0 == -1

        x.int_i2_0 = -2

        assert x.int_i2_0 == -2

        result = x.check_int_i2_0()

        assert result.result

        assert x.int_i2_1 == 0

        x.int_i2_1 = 0

        assert x.int_i2_1 == 0

        result = x.check_int_i2_1()

        assert result.result

        assert x.int_i2_2 == 1

        x.int_i2_2 = 2

        assert x.int_i2_2 == 2

        result = x.check_int_i2_2()

        assert result.result

        assert x.int_i4_0 == -1

        x.int_i4_0 = -2

        assert x.int_i4_0 == -2

        result = x.check_int_i4_0()

        assert result.result

        assert x.int_i4_1 == 0

        x.int_i4_1 = 0

        assert x.int_i4_1 == 0

        result = x.check_int_i4_1()

        assert result.result

        assert x.int_i4_2 == 1

        x.int_i4_2 = 2

        assert x.int_i4_2 == 2

        result = x.check_int_i4_2()

        assert result.result

        assert x.int_i8_0 == -1

        x.int_i8_0 = -2

        assert x.int_i8_0 == -2

        result = x.check_int_i8_0()

        assert result.result

        assert x.int_i8_1 == 0

        x.int_i8_1 = 0

        assert x.int_i8_1 == 0

        result = x.check_int_i8_1()

        assert result.result

        assert x.int_i8_2 == 1

        x.int_i8_2 = 2

        assert x.int_i8_2 == 2

        result = x.check_int_i8_2()

        assert result.result

    def test_check_reals(self):
        assert x.real_r4_0 == -3.140000104904175

        x.real_r4_0 = -6.28000020980835

        assert x.real_r4_0 == -6.28000020980835

        result = x.check_real_r4_0()

        assert result.result

        assert x.real_r4_1 == 0

        x.real_r4_1 = 0

        assert x.real_r4_1 == 0

        result = x.check_real_r4_1()

        assert result.result

        assert x.real_r4_2 == 3.140000104904175

        x.real_r4_2 = 6.28000020980835

        assert x.real_r4_2 == 6.28000020980835

        result = x.check_real_r4_2()

        assert result.result

        assert x.real_r8_0 == -3.140000104904175

        x.real_r8_0 = -6.28000020980835

        assert x.real_r8_0 == -6.28000020980835

        result = x.check_real_r8_0()

        assert result.result

        assert x.real_r8_1 == 0

        x.real_r8_1 = 0

        assert x.real_r8_1 == 0

        result = x.check_real_r8_1()

        assert result.result

        assert x.real_r8_2 == 3.140000104904175

        x.real_r8_2 = 6.28000020980835

        assert x.real_r8_2 == 6.28000020980835

        result = x.check_real_r8_2()

        assert result.result
